Attribute VB_Name = "modHashFile"
Option Explicit

Public Declare Function CryptAcquireContext _
               Lib "advapi32.dll" _
               Alias "CryptAcquireContextA" (ByRef phProv As Long, _
                                             ByVal pszContainer As String, _
                                             ByVal pszProvider As String, _
                                             ByVal dwProvType As Long, _
                                             ByVal dwFlags As Long) As Long

Public Declare Function CryptReleaseContext _
               Lib "advapi32.dll" (ByVal hProv As Long, _
                                   ByVal dwFlags As Long) As Long

Public Declare Function CryptCreateHash _
               Lib "advapi32.dll" (ByVal hProv As Long, _
                                   ByVal Algid As Long, _
                                   ByVal hKey As Long, _
                                   ByVal dwFlags As Long, _
                                   ByRef phHash As Long) As Long

Public Declare Function CryptDestroyHash Lib "advapi32.dll" (ByVal hHash As Long) As Long

Public Declare Function CryptHashData _
               Lib "advapi32.dll" (ByVal hHash As Long, _
                                   pbData As Any, _
                                   ByVal dwDataLen As Long, _
                                   ByVal dwFlags As Long) As Long

Public Declare Function CryptGetHashParam _
               Lib "advapi32.dll" (ByVal hHash As Long, _
                                   ByVal dwParam As Long, _
                                   pbData As Any, _
                                   pdwDataLen As Long, _
                                   ByVal dwFlags As Long) As Long

Public Const PROV_RSA_FULL = 1

Public Const CRYPT_NEWKEYSET = &H8

Public Const ALG_CLASS_HASH = 32768

Public Const ALG_TYPE_ANY = 0

Public Const ALG_SID_MD2 = 1

Public Const ALG_SID_MD4 = 2

Public Const ALG_SID_MD5 = 3

Public Const ALG_SID_SHA1 = 4

Enum HashAlgorithm

    MD2 = ALG_CLASS_HASH Or ALG_TYPE_ANY Or ALG_SID_MD2
    MD4 = ALG_CLASS_HASH Or ALG_TYPE_ANY Or ALG_SID_MD4
    MD5 = ALG_CLASS_HASH Or ALG_TYPE_ANY Or ALG_SID_MD5
    SHA1 = ALG_CLASS_HASH Or ALG_TYPE_ANY Or ALG_SID_SHA1

End Enum

Public Const HP_HASHVAL = 2

Public Const HP_HASHSIZE = 4

Public Function HashFile(ByVal FileName As String, _
                         Optional ByVal Algorithm As HashAlgorithm = MD5) As String

    Dim hCtx     As Long

    Dim hHash    As Long

    Dim lFile    As Long

    Dim lRes     As Long

    Dim lLen     As Long

    Dim lIdx     As Long

    Dim abHash() As Byte
    
    DoEvents

    If Len(Dir$(FileName)) = 0 Then Err.Raise 53
    lRes = CryptAcquireContext(hCtx, vbNullString, vbNullString, PROV_RSA_FULL, 0)

    If lRes = 0 And Err.LastDllError = &H80090016 Then
        lRes = CryptAcquireContext(hCtx, vbNullString, vbNullString, PROV_RSA_FULL, CRYPT_NEWKEYSET)
    End If

    If lRes <> 0 Then
        lRes = CryptCreateHash(hCtx, Algorithm, 0, 0, hHash)

        If lRes <> 0 Then
            lFile = FreeFile
            Open FileName For Binary As lFile

            If Err.Number = 0 Then

                Const BLOCK_SIZE As Long = 32 * 1024& ' 32K

                ReDim abBlock(1 To BLOCK_SIZE) As Byte

                Dim lCount     As Long

                Dim lBlocks    As Long

                Dim lLastBlock As Long

                lBlocks = LOF(lFile) \ BLOCK_SIZE
                lLastBlock = LOF(lFile) - lBlocks * BLOCK_SIZE

                For lCount = 1 To lBlocks
                    Get lFile, , abBlock
                    lRes = CryptHashData(hHash, abBlock(1), BLOCK_SIZE, 0)

                    If lRes = 0 Then Exit For
                Next

                If lLastBlock > 0 And lRes <> 0 Then
                    ReDim abBlock(1 To lLastBlock) As Byte
                    Get lFile, , abBlock
                    lRes = CryptHashData(hHash, abBlock(1), lLastBlock, 0)
                End If

                Close lFile
            End If

            If lRes <> 0 Then
                lRes = CryptGetHashParam(hHash, HP_HASHSIZE, lLen, 4, 0)

                If lRes <> 0 Then
                    ReDim abHash(0 To lLen - 1)
                    lRes = CryptGetHashParam(hHash, HP_HASHVAL, abHash(0), lLen, 0)

                    If lRes <> 0 Then

                        For lIdx = 0 To UBound(abHash)
                            HashFile = HashFile & Right$("0" & Hex$(abHash(lIdx)), 2)

                            DoEvents
                        Next

                    End If
                End If
            End If

            CryptDestroyHash hHash
        End If
    End If

    CryptReleaseContext hCtx, 0

    If lRes = 0 Then Err.Raise Err.LastDllError
End Function

Public Function crc32file(sFilePath As String, Optional Block As Long = 1024) As String

    Dim CRC32Table(255) As Long

    Dim hFile           As Long, i As Long, iCRC As Long, lngA As Long, Size As Long, Ret As Long, t As Long, n As Long

    Dim bytT            As Byte, bytC As Byte

    Dim sSize           As Currency, total As Currency

    Dim intI            As Integer, j As Integer, Crc32 As Long, Temp As Long

    Dim Seed            As Long

    Dim Precondition    As Long
    
    DoEvents

    If Len(Dir(sFilePath)) = 0 Then Exit Function
    '    InitCrc32
    
    Seed = &HEDB88320
    Precondition = &HFFFFFFFF

    For intI = 0 To 255
        Crc32 = intI

        For j = 0 To 7
            Temp = ((Crc32 And &HFFFFFFFE) \ &H2) And &H7FFFFFFF

            If (Crc32 And &H1) Then Crc32 = Temp Xor Seed Else Crc32 = Temp
        Next

        CRC32Table(intI) = Crc32
    Next
    
    iCRC = &HFFFFFFFF
    total = FileLen(sFilePath)

    If total < 0 Then total = total + 256 ^ 4
    sSize = Block * 1024
    t = Int((total - 1) / sSize) + 1
    hFile = FreeFile
    Open sFilePath For Binary Access Read As #hFile

    While total > 0

        If total >= sSize Then Size = sSize Else Size = total
        ReDim b(Size - 1) As Byte
        Get #hFile, , b

        For i = 0 To UBound(b)
            bytC = b(i)
            bytT = (iCRC And &HFF) Xor bytC
            lngA = ((iCRC And &HFFFFFF00) \ &H100) And &HFFFFFF
            iCRC = lngA Xor CRC32Table(bytT)
        Next

        total = total - sSize: n = n + 1

    Wend

    Close #hFile
    Ret = iCRC Xor &HFFFFFFFF
    crc32file = Hex(Ret)
End Function
